"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DBManager = void 0;
const rlp = __importStar(require("rlp"));
const ethereumjs_util_1 = require("ethereumjs-util");
const block_1 = require("@ethereumjs/block");
const cache_1 = __importDefault(require("./cache"));
const util_1 = require("./util");
const level = require('level-mem');
/**
 * Abstraction over a DB to facilitate storing/fetching blockchain-related
 * data, such as blocks and headers, indices, and the head block.
 * @hidden
 */
class DBManager {
    constructor(db, common) {
        this._db = db;
        this._common = common;
        this._cache = {
            td: new cache_1.default({ max: 1024 }),
            header: new cache_1.default({ max: 512 }),
            body: new cache_1.default({ max: 256 }),
            numberToHash: new cache_1.default({ max: 2048 }),
            hashToNumber: new cache_1.default({ max: 2048 }),
        };
    }
    /**
     * Fetches iterator heads from the db.
     */
    async getHeads() {
        const heads = await this.get(util_1.HEADS_KEY, { valueEncoding: 'json' });
        Object.keys(heads).forEach((key) => {
            heads[key] = Buffer.from(heads[key]);
        });
        return heads;
    }
    /**
     * Fetches header of the head block.
     */
    async getHeadHeader() {
        return this.get(util_1.HEAD_HEADER_KEY);
    }
    /**
     * Fetches head block.
     */
    async getHeadBlock() {
        return this.get(util_1.HEAD_BLOCK_KEY);
    }
    /**
     * Fetches a block (header and body) given a block id,
     * which can be either its hash or its number.
     */
    async getBlock(blockId) {
        if (typeof blockId === 'number' && Number.isInteger(blockId)) {
            blockId = new ethereumjs_util_1.BN(blockId);
        }
        let number;
        let hash;
        if (Buffer.isBuffer(blockId)) {
            hash = blockId;
            number = await this.hashToNumber(blockId);
        }
        else if (ethereumjs_util_1.BN.isBN(blockId)) {
            number = blockId;
            hash = await this.numberToHash(blockId);
        }
        else {
            throw new Error('Unknown blockId type');
        }
        const header = (await this.getHeader(hash, number)).raw();
        let body = [[], []];
        try {
            body = await this.getBody(hash, number);
        }
        catch (error) {
            if (error.type !== 'NotFoundError') {
                throw error;
            }
        }
        const blockData = [header, ...body];
        const opts = { common: this._common };
        return block_1.Block.fromValuesArray(blockData, opts);
    }
    /**
     * Fetches body of a block given its hash and number.
     */
    async getBody(hash, number) {
        const key = util_1.bodyKey(number, hash);
        const body = await this.get(key, { cache: 'body' });
        return rlp.decode(body);
    }
    /**
     * Fetches header of a block given its hash and number.
     */
    async getHeader(hash, number) {
        const key = util_1.headerKey(number, hash);
        const encodedHeader = await this.get(key, { cache: 'header' });
        const opts = { common: this._common };
        return block_1.BlockHeader.fromRLPSerializedHeader(encodedHeader, opts);
    }
    /**
     * Fetches total difficulty for a block given its hash and number.
     */
    async getTd(hash, number) {
        const key = util_1.tdKey(number, hash);
        const td = await this.get(key, { cache: 'td' });
        return new ethereumjs_util_1.BN(rlp.decode(td));
    }
    /**
     * Performs a block hash to block number lookup.
     */
    async hashToNumber(hash) {
        const key = util_1.hashToNumberKey(hash);
        const value = await this.get(key, { cache: 'hashToNumber' });
        return new ethereumjs_util_1.BN(value);
    }
    /**
     * Performs a block number to block hash lookup.
     */
    async numberToHash(number) {
        if (number.ltn(0)) {
            throw new level.errors.NotFoundError();
        }
        const key = util_1.numberToHashKey(number);
        return this.get(key, { cache: 'numberToHash' });
    }
    /**
     * Fetches a key from the db. If `opts.cache` is specified
     * it first tries to load from cache, and on cache miss will
     * try to put the fetched item on cache afterwards.
     */
    async get(key, opts = {}) {
        const dbOpts = {
            keyEncoding: opts.keyEncoding || 'binary',
            valueEncoding: opts.valueEncoding || 'binary',
        };
        if (opts.cache) {
            if (!this._cache[opts.cache]) {
                throw new Error(`Invalid cache: ${opts.cache}`);
            }
            let value = this._cache[opts.cache].get(key);
            if (!value) {
                value = await this._db.get(key, dbOpts);
                this._cache[opts.cache].set(key, value);
            }
            return value;
        }
        return this._db.get(key, dbOpts);
    }
    /**
     * Performs a batch operation on db.
     */
    async batch(ops) {
        return this._db.batch(ops);
    }
}
exports.DBManager = DBManager;
//# sourceMappingURL=dbManager.js.map